<?php

namespace Drupal\index_now\Service;

use Drupal\Core\Config\ConfigFactory;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Messenger\Messenger;
use Drupal\Core\Url;
use GuzzleHttp\Client;

/**
 * Index now class.
 */
class IndexNow implements IndexNowInterface {

  const BING_URL = 'https://www.bing.com';

  const YANDEX_URL = 'https://yandex.com';

  /**
   * The Index now API key.
   *
   * @var string
   */
  protected $apiKey;

  /**
   * The Index now default engine.
   *
   * @var string
   */
  protected $defaultEngine;

  /**
   * The HTTP client service.
   *
   * @var \GuzzleHttp\Client
   */
  protected $httpClient;

  /**
   * The logger service.
   *
   * @var \Drupal\Core\Logger\LoggerChannelFactoryInterface
   */
  protected $logger;

  /**
   * The messenger service.
   *
   * @var \Drupal\Core\Messenger\Messenger
   */
  protected $messenger;

  /**
   * IndexNow constructor.
   *
   * @param \GuzzleHttp\Client $http_client
   *   The HTTP client service.
   * @param \Drupal\Core\Config\ConfigFactory $config_factory
   *   The config factory.
   * @param \Drupal\Core\Messenger\Messenger $messenger
   *   The messenger service.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_channel_factory
   *   The logger service.
   */
  public function __construct(
    Client $http_client,
    ConfigFactory $config_factory,
    Messenger $messenger,
    LoggerChannelFactoryInterface $logger_channel_factory
  ) {
    $this->httpClient = $http_client;
    $this->apiKey = $config_factory
      ->getEditable('index_now.settings')
      ->get('apikey');
    $this->defaultEngine = $config_factory
      ->getEditable('index_now.settings')
      ->get('default_engine');
    $this->logger = $logger_channel_factory;
    $this->messenger = $messenger;
  }

  /**
   * {@inheritdoc}
   */
  public function sendRequest(string $page_url) {
    $url = $this->buildUrl($page_url);
    $url = urldecode($url->toString());

    try {
      $request = $this->httpClient->request('GET', $url);
      if ($request->getStatusCode() != 200) {
        $warning = "Index Now returned an error: " . $request->getContent();
        $this->messenger->addWarning($warning);
        $this->logger->get('index_now')->warning($warning);
        return;
      }
      $this->logger->get('index_now')->info($page_url . " has been indexed now.");
    }
    catch (\Exception $e) {
      $warning = "Index Now returned an error: " . $e->getMessage();
      $this->messenger->addWarning($warning);
      $this->logger->get('index_now')->warning($warning);
    }
  }

  /**
   * Build the search engine URL we want to hit to index now.
   *
   * @param string $page_url
   *   The page we want to index now on the search engines.
   *
   * @return \Drupal\Core\Url
   *   The search engine URL we want to hit to index now.
   */
  protected function buildUrl(string $page_url) {
    switch ($this->defaultEngine) {
      case 'yandex':
        $engine_url = self::YANDEX_URL;
        break;

      case 'bing': default:
          $engine_url = self::BING_URL;
        break;
    }

    $endpoint = $engine_url . "/indexnow";
    $api_key_location = Url::fromUri('base:index_now_key.txt');
    $api_key_location->setAbsolute();

    $url = Url::fromUri($endpoint);
    $url->setOption('query', [
      'url' => $page_url,
      'key' => $this->apiKey,
      'keyLocation' => $api_key_location->toString(),
    ]);

    return $url;
  }

}
